// Written by Yanko
// Modified by TeTeT

if (!hasInterface) exitWith {};

// register with server

if (isServer) then {
   [ObjNull, player] call TTT_fnc_subscribeToRadar;
} else {
   TTT_PV_subscribeToRadar = player;
   publicVariableServer "TTT_PV_subscribeToRadar";
};

systemChat "Client waiting on TTT_radarCtrl";
waitUntil {!isNil "TTT_radarCtrl"};
systemChat "Client waiting on TTT_radarAllObjects";
waitUntil {!isNil "TTT_radarAllObjects"};

params [ ["_refresh", 1], ["_updateFreq", 10] ];

private ["_marker", "_objects", "_allObjects", "_i", "_j", "_unit", "_radar", "_names",
         "_trackedObjects", "_object", "_radars", "_markerNames", "_markerName",
         "_fnc_getTrackedObjects", "_fnc_showMarker", "_fnc_createMarkerAndName",
         "_trackedLong", "_trackedShort", "_shortRange", "_longRange"];

// guard
player setVariable ["TTT_radar", true];

_radars = TTT_radarCtrl getVariable ["TTT_radars", []];

// display range of radar
_markerNames = [];
{
   _radar = _x;
   _shortRange = _radar getVariable ["TTT_radarShortRange", 1000];
   _longRange = _radar getVariable ["TTT_radarLongRange", 4000];
   _names = [];
   _markerName = format ["%1_shortRange", _radar];
   _names pushBack _markerName;
   _marker = createMarkerLocal [_markerName, position _radar];
   _marker setMarkerShapeLocal "ELLIPSE";
   _marker setMarkerSizeLocal [_shortRange, _shortRange];
   _marker setMarkerColorLocal "ColorRed";
   _marker setMarkerAlphaLocal 0.2;

   _markerName = format ["%1_longRange", _radar];
   _names pushBack _markerName;
   _marker = createMarkerLocal [_markerName, position _radar];
   _marker setMarkerShapeLocal "ELLIPSE";
   _marker setMarkerSizeLocal [_longRange, _longRange];
   _marker setMarkerColorLocal "ColorGreen";
   _marker setMarkerAlphaLocal 0.2;

   _markerNames pushBack _names;
} forEach _radars;

systemChat str _markerNames;

_fnc_showMarker = {
   params ["_unitArray", "_mode"];
   private ["_unit", "_marker", "_objName"];
   _unit = _unitArray select 0;
   _marker = _unitArray select 1;
   _objName = _unitArray select 2;

   switch (_mode) do {
      case "short" : {
         _marker setMarkerTextlocal format ["%1 A:%2 S:%3 D:%4", _objName, round(getPosASL _unit select 2), round speed _unit, round getdir _unit];
      };
      case "long" : {
         _marker setMarkerTextlocal format ["%1", _objName];
      };
      default {
         _marker setMarkerAlphaLocal 0;
      };
   };
   _marker setMarkerPosLocal getPos _unit;
};

_fnc_createMarkerAndName = {
   params ["_unit"];
   private ["_marker", "_objName"];
   _marker = createMarkerLocal [format ["m%1",_unit], position _unit];
   _marker setMarkerShapeLocal "ICON";
   _marker setMarkerTypeLocal "mil_dot";
   if (side _unit == side player) then {
      _marker setMarkerColorLocal "COLORBLUE";
   } else {
      _marker setMarkerColorLocal "COLORRED";
   };

   _cfg = (configFile >> "CfgVehicles" >> typeOf _unit);
   _objName = getText(_cfg >> "displayName");

   // Combining all in one array
   [_unit, _marker, _objName];
};


//main loop
while {alive player and !isNil{player getVariable "TTT_radar"}} do {

   // update radar markers
   {
      (_markerNames select _forEachIndex select 0) setMarkerPosLocal getPos _x;
      (_markerNames select _forEachIndex select 1) setMarkerPosLocal getPos _x;
   } forEach _radars;

   //create marker matrix

   _allObjects = TTT_radarAllObjects;

   _trackedLong = [];
   _trackedShort = [];

   {
      _object = [_x] call _fnc_createMarkerAndName;
      _trackedShort pushBack _object;
   } forEach (_allObjects select 0);

   {
      _object = [_x] call _fnc_createMarkerAndName;
      _trackedLong pushBack _object;
   } forEach (_allObjects select 1);

   //update position of markers
   _j = 0;
   while {_j < _updateFreq and !isNil{player getVariable "TTT_radar"}} do {
      // track units on radar
      {
         /*systemChat str _x;*/
         [_x, "short"] call _fnc_showMarker;
      } forEach _trackedShort;
      {
         /*systemChat str _x;*/
         [_x, "long"] call _fnc_showMarker;
      } forEach _trackedLong;

      sleep _refresh;
      _j =_j + 1;
   };

   // delete markers
   {
      deleteMarkerLocal (_x select 1);
   } forEach _trackedShort + _trackedLong;
};

systemChat "Shutting down radar";
systemChat str _markerNames;
{
   {
      deleteMarkerLocal _x;
   } forEach _x;
} forEach _markerNames;

/*systemChat "Radar script stopped";*/
player setVariable ["TTT_radar", nil];

if (isServer) then {
   [ObjNull, player] call TTT_fnc_unsubscribeFromRadar;
} else {
   TTT_PV_unsubscribeFromRadar = player;
   publicVariableServer "TTT_PV_unsubscribeFromRadar";
};